@@ -22,87 +22,184 @@ from __future__ import with_statement
 import sys
 import datetime
 import os
-import zipfile
 import operator
 import collections
 import time
+from optparse import OptionParser, OptionValueError
+
+
+	
 
 
 if hasattr(sys, "frozen"):
 	from rednotebook.util import filesystem
 	from rednotebook.util import utils
+	from rednotebook import info
 else:
 	from util import filesystem # creates a copy of filesystem module
 	#import util.filesystem # imports the original filesystem module
 	from util import utils
-
-## Enable logging
-import logging
-loggingLevels = {'debug': logging.DEBUG,
-				'info': logging.INFO,
-				'warning': logging.WARNING,
-				'error': logging.ERROR,
-				'critical': logging.CRITICAL}
-
-
-# Assert that all dirs and files are in place so that logging can take start
-filesystem.makeDirectories([filesystem.redNotebookUserDir, filesystem.dataDir, \
-							filesystem.templateDir, filesystem.tempDir])
-filesystem.makeFiles([(filesystem.configFile, ''),
-						(filesystem.logFile, '')])
-
-
-# File logging
-if sys.platform == 'win32' and hasattr(sys, "frozen"):
-	utils.redirect_output_to_file()
+	import info
 	
 
+def parse_options():
+	parser = OptionParser(usage="usage: %prog [options] [journal-path]",
+						  description=info.command_line_help,
+						  #option_class=ExtOption,
+						  formatter=utils.IndentedHelpFormatterWithNL(),
+						  )
+#	parser.add_option(
+#		'-p', '--portable', dest='portable', default=False,
+#		action='store_true', 
+#		help='Run in portable mode ' \
+#			'(default: False)')
+	
+	parser.add_option(
+		'-d', '--debug', dest='debug', \
+		default=False, action='store_true',
+		help='Output debugging messages ' \
+			'(default: False)')
+	
+	parser.add_option(
+		'-m', '--minimized', dest='minimized', \
+		default=False, action='store_true',
+		help='Start mimimized to system tray ' \
+			'(default: False)')
+	
+	options, args = parser.parse_args()
 		
+	return options, args
 
-file_logging_stream = open(filesystem.logFile, 'w')
-
-# We want to have the error messages in the logfile
-sys.stderr = utils.StreamDuplicator(sys.__stderr__, [file_logging_stream])
+options, args = parse_options()
 
 
-# Write a log containing every output to a log file
-logging.basicConfig(level=logging.DEBUG,
-					format='%(asctime)s %(levelname)-8s %(message)s',
-					#filename=filesystem.logFile,
-					#filemode='w',
-					stream=file_logging_stream,#sys.stdout,
-					)
 
-level = logging.INFO
-if len(sys.argv) > 1:
-	level = loggingLevels.get(sys.argv[1], level)
+## ---------------------- Enable logging -------------------------------
+
+import logging
+
+def setup_logging(log_file):
+	loggingLevels = {'debug': logging.DEBUG,
+					'info': logging.INFO,
+					'warning': logging.WARNING,
+					'error': logging.ERROR,
+					'critical': logging.CRITICAL}
 	
-logging.debug('sys.stdout logging level: %s' % level)
+	# File logging
+	if sys.platform == 'win32' and hasattr(sys, "frozen"):
+		utils.redirect_output_to_file(log_file)
+	
+	file_logging_stream = open(log_file, 'w')
+	
+	# We want to have the error messages in the logfile
+	sys.stderr = utils.StreamDuplicator(sys.__stderr__, [file_logging_stream])
+	
+	# Write a log containing every output to a log file
+	logging.basicConfig(level=logging.DEBUG,
+						format='%(asctime)s %(levelname)-8s %(message)s',
+						#filename=filesystem.logFile,
+						#filemode='w',
+						stream=file_logging_stream,#sys.stdout,
+						)
+	
+	level = logging.INFO
+	#if len(sys.argv) > 1:
+		#level = loggingLevels.get(sys.argv[1], level)
+	if options.debug:
+		level = logging.DEBUG
+	
+	# define a Handler which writes INFO messages or higher to the sys.stdout
+	console = logging.StreamHandler(sys.stdout)
+	console.setLevel(level)
+	# set a format which is simpler for console use
+	formatter = logging.Formatter('%(levelname)-8s %(message)s')
+	# tell the handler to use this format
+	console.setFormatter(formatter)
+	# add the handler to the root logger
+	logging.getLogger('').addHandler(console)
+	
+	logging.debug('sys.stdout logging level: %s' % level)
+	logging.info('Writing log to file "%s"' % log_file)
+
+
+dirs = filesystem.Filenames()
+setup_logging(dirs.logFile)
+
+## ------------------ end Enable logging -------------------------------
+
+
+
+## ---------------------- Enable i18n -------------------------------
+
+# set the locale for all categories to the user’s default setting 
+# (typically specified in the LANG environment variable)
+import locale
+try:
+	locale.setlocale(locale.LC_ALL, '')
+except locale.Error, err:
+	# unsupported locale setting
+	lang = os.getenv('LANG')
+	logging.error('Locale "%s" could not be set: "%s"' % (lang, err))
+	logging.error('Probably you have to install the appropriate language packs')
+
+LOCALE_PATH = os.path.join(dirs.appDir, 'i18n')
+
+# the name of the gettext domain. because we have our translation files
+# not in a global folder this doesn't really matter, setting it to the
+# application name is a good idea tough.
+GETTEXT_DOMAIN = 'rednotebook'
+
+# set up the gettext system
+import gettext
+
+modules = [gettext]
+
+# Little Hack: The C function bindtextdomain() has to be called for
+# gtkbuilder stuff to be translated. This is not yet done by gtkbuilder.
+# As a workaround we use gtk.glade.bindtextdomain
+try:
+	import gtk.glade
+	modules.append(gtk.glade)
+except ImportError, err:
+	logging.warning('Importing gtk.glade failed. Some strings may not be translated')
+
+for module in modules:
+    module.bindtextdomain(GETTEXT_DOMAIN, LOCALE_PATH)
+    module.textdomain(GETTEXT_DOMAIN)
+    
+
+
+
+# register the gettext function for the whole interpreter as "_"
+import __builtin__
+__builtin__._ = gettext.gettext
+
+## ------------------- end Enable i18n -------------------------------
+
 
-# define a Handler which writes INFO messages or higher to the sys.stdout
-console = logging.StreamHandler(sys.stdout)
-console.setLevel(level)
-# set a format which is simpler for console use
-formatter = logging.Formatter('%(levelname)-8s %(message)s')
-# tell the handler to use this format
-console.setFormatter(formatter)
-# add the handler to the root logger
-logging.getLogger('').addHandler(console)
 
 
 try:
 	import pygtk
 except ImportError:
-	logging.error('Please install PyGTK (python-gtk2)')
+	logging.error('pygtk not found. Please install PyGTK (python-gtk2)')
 	sys.exit(1)
 
 pygtk.require("2.0")
 
 try:
 	import gtk
+	#logging.disable(logging.DEBUG)
+	#logging.disable(logging.INFO)
+	#logging.disable(logging.ERROR)
+	#logging.disable(logging.WARNING)
+	#logging.disable(logging.CRITICAL)
+	gtk.gdk.threads_init()
+	#gtk.gdk.threads_enter()
+	
 	import gobject
 except (ImportError, AssertionError):
-	logging.error('Please install PyGTK (python-gtk2)')
+	logging.error('gtk not found. Please install PyGTK (python-gtk2)')
 	sys.exit(1)
 
 try:
@@ -135,7 +232,7 @@ if baseDir not in sys.path:
 # This version of import is needed for win32 to work
 from rednotebook.util import unicode
 from rednotebook.util import dates
-from rednotebook import info
+#from rednotebook import info
 from rednotebook import config
 from rednotebook import backup
 
@@ -147,10 +244,21 @@ from rednotebook.util.statistics import Statistics
 class RedNotebook:
 	
 	def __init__(self):
+		self.dirs = dirs
+		
+		self.config = config.Config(self.dirs)
+		logging.info('Running in portable mode: %s' % self.config.read('portable', 0))
+		
 		self.testing = False
-		if 'debug' in sys.argv:
+		if options.debug:
 			self.testing = True
 			logging.debug('Debug Mode is on')
+			
+		# Allow starting minimized to tray
+		# When we start minimized we have to set the tray icon visible
+		self.start_minimized = options.minimized
+		if self.start_minimized:
+			self.config['closeToTray'] = 1
 		
 		self.month = None
 		self.date = None
@@ -159,17 +267,15 @@ class RedNotebook:
 		# The dir name is the title
 		self.title = ''
 		
-		self.dirs = filesystem.Filenames()
-		
 		# show instructions at first start or if testing
-		self.firstTimeExecution = len(os.listdir(self.dirs.dataDir)) == 0
+		default_dir_selected = (self.dirs.dataDir == self.dirs.defaultDataDir)
+		self.firstTimeExecution = default_dir_selected and \
+									len(os.listdir(self.dirs.dataDir)) == 0
 		logging.info('First Start: %s' % self.firstTimeExecution)
 		
 		logging.info('RedNotebook version: %s' % info.version)
 		logging.info(filesystem.get_platform_info())
 		
-		self.config = config.Config()
-		
 		utils.set_environment_variables(self.config)
 		
 		# Let components check if the MainWindow has been created
@@ -178,16 +284,7 @@ class RedNotebook:
 		   
 		self.actualDate = datetime.date.today()
 		
-		self.dirs.dataDir = self.config.read('dataDir', self.dirs.dataDir)
-		
-		if self.testing:
-			self.dirs.dataDir = os.path.join(filesystem.redNotebookUserDir, "data-test/")
-			filesystem.makeDirectory(self.dirs.dataDir)
-		# HACK: Only load test dir with active debug option
-		elif self.dirs.dataDir == os.path.join(filesystem.redNotebookUserDir, "data-test/"):
-			self.dirs.dataDir = filesystem.defaultDataDir
-			
-		self.open_journal(self.dirs.dataDir)
+		self.open_journal(self.get_journal_path())
 		
 		self.archiver = backup.Archiver(self)
 		
@@ -196,10 +293,41 @@ class RedNotebook:
 			utils.check_new_version(self.frame, info.version, startup=True)
 			
 		# Automatically save the content after a period of time
-		one_minute = 1000 * 60
-		
 		if not self.testing:
-			gobject.timeout_add(10 * one_minute, self.saveToDisk)
+			gobject.timeout_add_seconds(600, self.saveToDisk)
+			
+	
+	def get_journal_path(self):
+		'''
+		Retrieve the path from optional args or return standard value if args
+		not present
+		'''
+		if not args:
+			return self.config.read('dataDir', self.dirs.dataDir)
+		
+		# path_arg can be e.g. data (under .rednotebook), data (elsewhere), 
+		# or an absolute path /home/username/myjournal
+		# Try to find the journal under the standard location or at the given
+		# absolute or relative location
+		path_arg = args[0]
+		
+		logging.debug('Trying to find journal "%s"' % path_arg)
+		
+		paths_to_check = [path_arg, os.path.join(self.dirs.redNotebookUserDir, path_arg)]
+		
+		for path in paths_to_check:
+			if os.path.exists(path):
+				if os.path.isdir(path):
+					return path
+				else:
+					logging.warning('To open a journal you must specify a '
+								'directory, not a file.')
+		
+		logging.error('The path "%s" is no valid journal directory. ' 
+					'Execute "rednotebook -h" for instructions' % path_arg)
+		sys.exit(1)
+			
+			
 	
 	
 	def getDaysInDateRange(self, range):
@@ -236,20 +364,42 @@ class RedNotebook:
 		if backup_file:
 			self.archiver.backup(backup_file)
 			
+		
 	def exit(self):
 		self.frame.add_values_to_config()
-		self.saveToDisk()
-		logging.info('Goodbye!')
-		gtk.main_quit()
+		
+		# Make it possible to stop the program from exiting
+		# e.g. if the journal could not be saved
+		self.is_allowed_to_exit = True
+		self.saveToDisk(exitImminent=True)
+		
+		if self.is_allowed_to_exit:
+			logging.info('Goodbye!')
+			gtk.main_quit()
 
 	
-	def saveToDisk(self, exitImminent=False, changing_journal=False):
+	def saveToDisk(self, exitImminent=False, changing_journal=False, saveas=False):
+		logging.info('Trying to save the journal')
+		
 		self.saveOldDay()
 		
-		filesystem.makeDirectories([filesystem.redNotebookUserDir, self.dirs.dataDir,])
+		if not os.path.exists(self.dirs.dataDir):
+			logging.error('Save path does not exist')
+			self.frame.show_save_error_dialog(exitImminent)
+			return True
+			
+		try:
+			filesystem.makeDirectory(self.dirs.dataDir)
+		except OSError, err:
+			self.frame.show_save_error_dialog(exitImminent)
+			return True
+		
+		something_saved = False
 		
 		for yearAndMonth, month in self.months.items():
-			if not month.empty and month.visited:
+			# We always need to save everything when we are "saving as"
+			if (not month.empty and month.edited) or saveas:
+				something_saved = True
 				monthFileString = os.path.join(self.dirs.dataDir, yearAndMonth + \
 											filesystem.fileNameExtension)
 				with open(monthFileString, 'w') as monthFile:
@@ -258,14 +408,31 @@ class RedNotebook:
 						# do not add empty days
 						if not day.empty:
 							monthContent[dayNumber] = day.content
-					#month.prettyPrint()
-					yaml.dump(monthContent, monthFile, Dumper=Dumper)
+					
+					try:
+						yaml.dump(monthContent, monthFile, Dumper=Dumper)
+						month.edited = False
+					except OSError, err:
+						self.frame.show_save_error_dialog(exitImminent)
+						return True	
+					except IOError, err:
+						self.frame.show_save_error_dialog(exitImminent)
+						return True	
+					
 		
-		self.showMessage('The content has been saved to %s' % self.dirs.dataDir, error=False)
+		if something_saved:
+			self.showMessage(_('The content has been saved to %s') % self.dirs.dataDir, error=False)
+		else:
+			self.showMessage(_('Nothing to save'), error=False)
 		
-		self.config.saveToDisk()
+		if self.config.changed():
+			try:
+				filesystem.makeDirectory(self.dirs.redNotebookUserDir)
+				self.config.saveToDisk()
+			except IOError, err:
+				self.showMessage(_('Configuration could not be saved. Please check your permissions'))
 		
-		if not (exitImminent or changing_journal):
+		if not (exitImminent or changing_journal) and something_saved:
 			# Update cloud
 			self.frame.cloud.update(force_update=True)
 			
@@ -277,6 +444,9 @@ class RedNotebook:
 		
 		if self.months:
 			self.saveToDisk(changing_journal=True)
+			
+		# Password Protection
+		#password = self.config.read('password', '')
 		
 		logging.info('Opening journal at %s' % data_dir)
 		
@@ -290,12 +460,12 @@ class RedNotebook:
 		data_dir_empty = not os.listdir(data_dir)
 		
 		if not load_files and not data_dir_empty:
-			self.showMessage('The selected folder is not empty. To prevent ' + \
-							'you from overwriting data, the content has been ' + \
-							'imported into the new journal.', error=False)
+			msg_part1 = _('The selected folder is not empty.')
+			msg_part2 = _('To prevent you from overwriting data, the folder content has been imported into the new journal.')
+			self.showMessage('%s %s' % (msg_part1, msg_part2), error=False)
 		elif load_files and data_dir_empty:
-			self.showMessage('The selected folder is empty. A new journal ' + \
-							'has been created.', error=False)
+			self.showMessage(_('The selected folder is empty. A new journal has been created.'), \
+								error=False)
 		
 		self.dirs.dataDir = data_dir
 		
@@ -314,7 +484,8 @@ class RedNotebook:
 		sortedCategories = sorted(self.nodeNames, key=lambda category: str(category).lower())
 		self.frame.categoriesTreeView.categories = sortedCategories
 		
-		if self.firstTimeExecution is True:
+		if self.firstTimeExecution:
+			logging.info('Adding example content')
 			self.addInstructionContent()
 			
 		# Notebook is only on page 1 here, if we are opening a journal the second time
@@ -340,13 +511,22 @@ class RedNotebook:
 		# Save the folder for next start
 		self.config['dataDir'] = data_dir
 		
+		# Set the date range for the export assistant
+		start_date = self.getEditDateOfEntryNumber(0)
+		self.frame.export_assistant.set_start_date(start_date)
+
+		end_date = self.getEditDateOfEntryNumber(-1)
+		self.frame.export_assistant.set_end_date(end_date)
+		
 		
 		
 	def loadAllMonthsFromDisk(self):
 		logging.debug('Starting to load files in dir "%s"' % self.dirs.dataDir)
-		for root, dirs, files in os.walk(self.dirs.dataDir):
-			for file in files:
-				self.loadMonthFromDisk(os.path.join(root, file))
+		#for root, dirs, files in os.walk(self.dirs.dataDir):
+		files = os.listdir(self.dirs.dataDir)
+		for file in files:
+			if not file.endswith('~'):
+				self.loadMonthFromDisk(os.path.join(self.dirs.dataDir, file))
 		logging.debug('Finished loading files in dir "%s"' % self.dirs.dataDir)
 	
 	
@@ -376,6 +556,9 @@ Filenames have to have the following form: 2009-01.txt \
 			with open(monthFileString, 'r') as monthFile:
 				logging.debug('Start loading file "%s"' % monthFileString)
 				monthContents = yaml.load(monthFile, Loader=Loader)
+				#print monthContents
+				#monthContents = unicode.get_unicode_dict(monthContents)
+				#print monthContents
 				logging.debug('Finished loading file "%s"' % monthFileString)
 				self.months[yearAndMonth] = Month(yearNumber, monthNumber, monthContents)
 		except yaml.YAMLError, exc:
@@ -391,7 +574,7 @@ Filenames have to have the following form: 2009-01.txt \
 		
 		yearAndMonth = dates.getYearAndMonthFromDate(date)
 		
-		'Selected month has not been loaded or created yet'
+		# Selected month has not been loaded or created yet
 		if not self.months.has_key(yearAndMonth):
 			self.months[yearAndMonth] = Month(date.year, date.month)
 			
@@ -399,10 +582,15 @@ Filenames have to have the following form: 2009-01.txt \
 	
 	
 	def saveOldDay(self):
-		'Order is important'
+		'''Order is important'''
+		old_content = self.day.content
 		self.day.content = self.frame.categoriesTreeView.get_day_content()
-		
 		self.day.text = self.frame.get_day_text()
+		
+		content_changed = not (old_content == self.day.content)
+		if content_changed:
+			self.month.edited = True
+		
 		self.frame.calendar.setDayEdited(self.date.day, not self.day.empty)
 	
 	
@@ -412,7 +600,8 @@ Filenames have to have the following form: 2009-01.txt \
 		
 		if not Month.sameMonth(newDate, oldDate) or self.month is None:
 			self.month = self.loadMonth(self.date)
-			self.month.visited = True
+			#self.month.visited = True
+		
 		self.frame.set_date(self.month, self.date, self.day)
 		
 		
@@ -479,11 +668,15 @@ Filenames have to have the following form: 2009-01.txt \
 	
 	
 	def _getAllEditedDays(self):
+		# The day being edited counts too
+		if self.frame:
+			self.saveOldDay()
+			
 		days = []
 		for month in self.months.values():
 			daysInMonth = month.days.values()
 			
-			'Filter out days without content'
+			# Filter out days without content
 			daysInMonth = filter(lambda day: not day.empty, daysInMonth)
 			days.extend(daysInMonth)
 		return days
@@ -506,18 +699,28 @@ Filenames have to have the following form: 2009-01.txt \
 			for word in words:
 				wordDict[word.lower()] += 1
 		return wordDict
+	
+	def go_to_first_empty_day(self):
+		if len(self.sortedDays) == 0:
+			return datetime.date.today()
+		
+		last_edited_day = self.sortedDays[-1]
+		first_empty_date = last_edited_day.date + dates.oneDay
+		self.changeDate(first_empty_date)
 			
 	
 	def addInstructionContent(self):
-		instructionDayContent = {u'Cool Stuff': {u'Went to see the pope': None}, 
-								 u'Ideas': {u'Invent Anti-Hangover-Machine': None},
-								 u'Tags': {u'Work': None, u'Projects': None},
-								 }
+		self.go_to_first_empty_day()
+		current_date = self.date
+		
+		for example_day in info.example_content:
+			self.day.content = example_day
+			self.frame.set_date(self.month, self.date, self.day)
+			self.goToNextDay()
+		
+		self.changeDate(current_date)
 		
-		self.day.content = instructionDayContent
-		self.day.text = info.completeWelcomeText
 		
-		self.frame.set_date(self.month, self.date, self.day)
 
 			
 
@@ -538,7 +741,7 @@ class Day(object):
 		return getattr(self.date, name)
 	
 	
-	'Text'
+	# Text
 	def _getText(self):
 		'''
 		Returns the day's text encoded as UTF-8
@@ -631,7 +834,7 @@ class Day(object):
 		occurence = upCaseDayText.find(upCaseSearchText)
 		
 		if occurence > -1:
-			'searchText is in text'
+			# searchText is in text
 			
 			searchedStringInText = self.text[occurence:occurence + len(searchText)]
 			
@@ -640,26 +843,28 @@ class Day(object):
 									occurence + len(searchText) + self.searchResultLength/2)
 				
 			resultTextStart = self.text.find(' ', spaceSearchLeftStart, occurence)
-			resultTextEnd = self.text.rfind(' ', occurence + len(searchText), spaceSearchRightEnd)
+			resultTextEnd = self.text.rfind(' ', \
+						occurence + len(searchText), spaceSearchRightEnd)
 			if resultTextStart == -1:
 				resultTextStart = occurence - self.searchResultLength/2
 			if resultTextEnd == -1:
 				resultTextEnd = occurence + len(searchText) + self.searchResultLength/2
 				
-			'Add leading and trailing ... if appropriate'
+			# Add leading and trailing ... if appropriate
 			resultText = ''
 			if resultTextStart > 0:
 				resultText += '... '
 				
 			resultText += unicode.substring(self.text, resultTextStart, resultTextEnd).strip()
 			
-			'Make the searchedText bold'
-			resultText = resultText.replace(searchedStringInText, '<b>' + searchedStringInText + '</b>')
+			# Make the searchedText bold
+			resultText = resultText.replace(searchedStringInText, \
+									'<b>' + searchedStringInText + '</b>')
 			
 			if resultTextEnd < len(self.text) - 1:
 				resultText += ' ...'
 				
-			'Delete newlines'
+			# Delete newlines
 			resultText = resultText.replace('\n', '')
 				
 			return (str(self), resultText)
@@ -684,7 +889,7 @@ class Day(object):
 					firstWhitespace = self.text.find(' ', self.searchResultLength)
 					
 					if firstWhitespace == -1:
-						'No whitespace found'
+						# No whitespace found
 						textStart = self.text
 					else:
 						textStart = self.text[:firstWhitespace + 1]
@@ -717,7 +922,7 @@ class Month(object):
 		for dayNumber, dayContent in monthContent.iteritems():
 			self.days[dayNumber] = Day(self, dayNumber, dayContent)
 			
-		self.visited = False
+		self.edited = False
 	
 	
 	def getDay(self, dayNumber):
@@ -785,7 +990,7 @@ def main():
 		#logging.debug('Closing logfile')
 		#file_logging_stream.close()
 	except KeyboardInterrupt:
-		#print 'Interrupt'
+		# 'Interrupt'
 		#redNotebook.saveToDisk()
 		sys.exit()
 		
